import pygame
from pygame.locals import *
import data
import physics

def make_platform(tiles):
    surface = pygame.Surface((len(tiles)*32, 32))
    position = 0
    for tile in tiles:
        surface.blit(tile.image, (position, 0))
        position += 32
    return Platform(surface, tiles[0].rect.topleft)

def level_loader(level):
    level = __import__(level)
    level_group = LevelGroup()
    platform_group = PlatformGroup()
    row = column = 0
    tiles = []
    for char in level.level_string:
        if char == "\n":
            row += 32
            column = -32
        elif char == "-":
            laser = (column, row)
        elif char == "s":
            start = (column, row)
        elif char == "e":
            heli = Heli("heli.png", (column-301, row-223+32))
            platform_group.add(heli)
        elif char in level.tiles:
            tiles.append(Tile(level.tiles[char], (column, row)))
        elif tiles:
            platform_group.add(make_platform(tiles))
            tiles = []
        column += 32
    level_group.add(platform_group.sprites())
    return level_group, platform_group, start, laser, heli

def get_image_list(image, frame_width):
    image_list = []
    height = image.get_height()
    for frame in xrange(image.get_width()/frame_width):
        image_list.append(image.subsurface(Rect(frame*frame_width, 0,
                                           frame_width, height)))
    return image_list

def clear_callback(surf, rect):
        color = 0, 200, 255
        surf.fill(color, rect)

class World:
    def __init__(self, screen):
        self.screen = screen
        self.level, self.platforms, self.start, \
                    laser, self.heli = level_loader("level1")
        self.laser = DeathLaser(laser)
        self.level.add(self.laser)

    def add_hero(self, hero):
        self.hero = hero
        self.level.add(self.hero)

    def move(self, x, y):
        self.platforms.move(-x, y)
        self.laser.move(-x, y)

    def check_collisions(self):
        collisions = pygame.sprite.spritecollide(self.hero,
                                                 self.platforms,
                                                 False)
        for platform in collisions:
            self.hero.check_direction(platform.rect)

    def step(self):
        if self.hero.physics.x_movement > 0:
            self.hero.image = self.hero.right_walk.next()
        elif self.hero.physics.x_movement < 0:
            self.hero.image = self.hero.left_walk.next()
        self.laser.time_move()
        self.hero.physics.apply_gravity()
        self.hero.move()
        if self.heli.rect.colliderect(self.hero.rect):
            self.win()
        self.check_collisions()
        self.level.clear(self.screen, clear_callback)
        rects = self.level.draw(self.screen)
        pygame.display.update(rects)
        if self.laser.rect.colliderect(self.hero.rect):
            self.death()

    def death(self):
        raise SystemExit

    def win(self):
        raise SystemExit
        

class Tile(pygame.sprite.Sprite):
    def __init__(self, image_filename, place, size=(32,32)):
        pygame.sprite.Sprite.__init__(self)
        if image_filename:
            self.image = pygame.image.load(data.load(
                image_filename)).convert()
        self.rect = Rect(place, self.image.get_size())

    def move(self, x, y):
        self.rect.move_ip(x, y)

class LevelGroup(pygame.sprite.RenderUpdates):
    def clear_callback(self, surf, rect):
        color = 0, 23, 186
        surf.fill(color, rect)

class PlatformGroup(LevelGroup):
    distance = [0,0]
    def move(self, x, y):
        for tile in self.sprites():
            tile.move(x, y)

class Platform(Tile):
    def __init__(self, surface, place):
        pygame.sprite.Sprite.__init__(self)
        self.image = surface
        self.rect = self.image.get_rect(topleft=place)

class DeathLaser(pygame.sprite.Sprite):
    def __init__(self, place):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.Surface((640, 2))
        pygame.draw.line(self.image, (255,0,0), (0,0), (640,0), 2)
        self.rect = Rect(place, (640,2))
        self.accumulated = 0

    def time_move(self):
        self.accumulated -= 1
        if int(self.accumulated) == -1:
            self.rect.top -= 1
            self.accumulated = 0

    def move(self, x, y):
        self.rect.move_ip(0, y)

class Heli(Tile):
    def __init__(self, image_filename, place):
        Tile.__init__(self, image_filename, place)
        self.image.set_colorkey((255,255,255))
    

class Hero(Tile):
    def __init__(self, world, image_filename, place):
        self.world = world
        Tile.__init__(self, image_filename, place)
        self.world.move(-240 + self.rect.centerx,
                        -320+0.5*self.rect.centery)
        self.rect.center = (320, 240)
        self.image.set_colorkey((235, 9, 243))
        self.physics = physics.Hero(self.rect)
        hero_walk = pygame.image.load(data.load("hero_walk3.png")).convert()
        hero_walk.set_colorkey((235, 9, 243))
        self.right_walk = self.get_frame(get_image_list(hero_walk, 40))
        hero_left = pygame.transform.flip(hero_walk, True, False)
        self.left_walk = self.get_frame(get_image_list(hero_left, 40))

    def get_frame(self, frame_list):
        while True:
            for frame in frame_list:
                yield frame

    def check_direction(self, rect):
        """Checks movement and moves the screen accordingly.

        This method checks the relative movement of the hero
        to determine the direction of impact. The hero is
        then moved back to the boundary of the platform."""
        # The -24 allows for incoming velocity
        if self.physics.force < 0 and \
               (self.rect.bottom - 16 < rect.top <= self.rect.bottom):
            self.world.move(0, self.rect.bottom - rect.top)
            self.physics.force = 0
        elif self.physics.x_movement and \
           (rect.left > self.rect.left or \
           rect.right < self.rect.right):
            if self.physics.x_movement > 0:
                self.world.move(rect.left - self.rect.right, 0)
            else:
                self.world.move(rect.right - self.rect.left, 0)
        elif self.physics.force > 0:
            self.physics.force = -1

    def move(self):
        self.world.move(self.physics.x_movement,
                        self.physics.force)

    def jump(self):
        self.physics.jump()

    def move_back(self):
        self.world.move(-self.physics.x_movement,
                        0)
